-- AISMod.lua
-- author: Maro_Crafciarz
-- date: 08.04.2022
-- Copyright 2022
-- report errors or something similar to report.error@Maro_Crafciarz.de
 
AISMod = {}
local AISMod_mt = Class(AISMod)

g_isAISModDeveloper = false
g_hasAISPerformedDebug = false

AISMod.ENABLE_CONSOLE_COMMANDS_INTERFACE = true
AISMod.ENABLE_AIS = true

function AISMod:new(isClient, modDir)
	--local spec = self.spec_aisMod - funktioniert nicht :(

	-- do this to avoid creating to instances of the mod
	if g_aisMod ~= nil then
		return
	end

	-- apply metatable
	local self = {}
	setmetatable(self, AISMod_mt)

	self.baseDir = modDir
	self.isClient = isClient

	-- add this instance of AISMod as ModEventListener
	addModEventListener(self)

	return self
end;

-- handler if drivable get load
function AISMod:onLoad()
	--local spec = self.spec_aisMod - funktioniert nicht :(

	if self.isClient then
		resetSteers(self)
	end
end;

function AISMod:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self.isClient and AISMod.ENABLE_AIS then
		
		local drivable_isActive = self:getIsActive() and isActiveForInputIgnoreSelection;
    
		--															for avoiding error with steering
		if self.spec_drivable ~= nil and drivable_isActive and self.spec_drivable.steeringWheel ~= nil then
			local steerRot = getSteeringRotation(self.spec_drivable)

			-- place for debuging tables to find functions which are not in docs
			if g_isAISModDeveloper ~= nil and g_isAISModDeveloper and not g_hasAISPerformedDebug then
				DebugUtil.printTableRecursively(self.spec_drivable.steeringWheel);
				g_hasAISPerformedDebug = true
			end

			local isSteering = self.spec_drivable.hasSteerToTurnRightDirection or self.spec_drivable.hasSteerToTurnLeftDirection;

			if isSteering then
				-- mod checks for permmission if it could automaticly turn off this lights 
				-- if the steering wheel is nearly centered 
				--				- you cloud customize these numbers i decided to set these as defaults
				if self.spec_drivable.hasSteerToTurnRightDirection and steerRot >= -0.001 then
					self:setTurnLightState(Lights.TURNLIGHT_OFF)
					resetSteers(self)
				elseif self.spec_drivable.hasSteerToTurnLeftDirection and steerRot <= 0.001 then
					self:setTurnLightState(Lights.TURNLIGHT_OFF)
					resetSteers(self)
				end
			-- check turn dricections
			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_RIGHT then
				-- register custom attr for chekcs above on next update
				self.spec_drivable.hasSteerToTurnRightDirection = steerRot < -0.1;
			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_LEFT then
				-- register custom attr for chekcs above on next update
				self.spec_drivable.hasSteerToTurnLeftDirection = steerRot > 0.1;
			end
    	end
	end
end;	

-- get steering wheel node and select needed
function getSteeringRotation(spec_drivable)
	local _, rotation, _ = getRotation(spec_drivable.steeringWheel.node)
	return rotation
end;

function initAISMod()
	if g_aisMod == nil and g_currentModName == "FS22_Automatic_turn_signal_stop" then
		local aisMod = AISMod:new(g_dedicatedServerInfo == nil, g_currentModDirectory)

		if aisMod ~= nil then
			-- edit env vars
			--local env = getfenv(0)
			--env["g_aisMod"] = aisMod
			--setfenv(0, env)
			-- end
			getfenv(0)["g_aisMod"] = aisMod

			if aisMod.isClient then
				if AISMod.ENABLE_CONSOLE_COMMANDS_INTERFACE then
					source(g_currentModDirectory .. "src/ConsoleCommands.lua")
					aisMod.consoleCommands = ConsoleCommands:new(aisMod)
				end
			end

      		print(" Loaded Automatic Indicator Stop Modification successfully ")
		else
			print("Could not init mod(AutoIndicatorStop) successfully!")
		end
	else
		print("AISMod is already initialized or you haven't finished downloading it from the Giants-ModHub! "..
			"Please redownload and install it from https://www.farming-simulator.com/mod.php?mod_id=242755");
	end
end;

function AISMod:loadMap(name)
	-- linking Drivable events with this customs
	Drivable.onLoad = Utils.prependedFunction(Drivable.onLoad, AISMod.onLoad)
	Drivable.onUpdate = Utils.prependedFunction(Drivable.onUpdate, AISMod.onUpdate)
end;

function AISMod:deleteMap()
	-- if commands havn't been already deleted do this now
	if self.consoleCommands ~= nil then
		self.consoleCommands:delete()
	end

	-- delete self - edit env vars
	local env = getfenv(0)
	env["g_aisMod"] = nil
	getfenv(0)["g_aisMod"] = nil
	-- does not work: setfenv(0, env)
	-- end
end;

function delete()
	AISMod:deleteMap();
end;

function resetSteers(modSpec)
	-- reset drivable vehicle steers
	modSpec.spec_drivable.hasSteerToTurnLeftDirection = false
	modSpec.spec_drivable.hasSteerToTurnRightDirection = false
end; 

-- init modification
initAISMod()

-- @end script