-- ConsoleCommands.lua
-- author: Maro_Crafciarz
-- date: 08.04.2022
-- Copyright 2022
-- report errors or something similar to report.error@Maro_Crafciarz.de

ConsoleCommands = {};
local ConsoleCommands_mt = Class(ConsoleCommands)

function ConsoleCommands:new(parent)
	if g_aisMod == nil then
    	return
	end

	local self = {}
	setmetatable(self, ConsoleCommands_mt)

	-- set AISMod as parent to easily access it
	self.parent = parent

    -- add commands
	addConsoleCommand("activateAIS", "A console command for activateing automatic indicator stop", 
		"consoleCommandActivateAIS", self)
	addConsoleCommand("deactivateAIS", "A console command for deactivateing automatic indicator stop", 
		"consoleCommandDeactivateAIS", self)
	addConsoleCommand("getAISStatus", "Printing current status of the AIS-System", "consoleCommandStatus", self)

	return self;
end;

function ConsoleCommands:delete()
	removeConsoleCommand("deactivateAIS")
	removeConsoleCommand("activateAIS")
	removeConsoleCommand("getAISStatus")

	--[[powered if the main mod is not nil because then 
		the consoleCommands have to be set to nil]]
	if g_aisMod ~= nil then
		g_aisMod.consoleCommands = nil
	end
end;

function ConsoleCommands:consoleCommandActivateAIS()
	if AISMod.ENABLE_AIS ~= true then AISMod.ENABLE_AIS = true; print("AutoIndicatorStop: activated"); end;
end;

function ConsoleCommands:consoleCommandDeactivateAIS()
	if AISMod.ENABLE_AIS ~= false then AISMod.ENABLE_AIS = false; print("AutoIndicatorStop: deactivated"); end;
end;

function ConsoleCommands:consoleCommandStatus()
	local status = "deactive"

	if self.parent.ENABLE_AIS then
		status = "active"
	end

	print("AutoIndicatorStop: " .. status);
end;

-- @end script